#!/bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/apache2-mod_brotli.sh
#
# Creation   :  2019-01-08 se, sebastian(dot)ertz(at)tk(minus)ertz(dot)de
# Last Update:  2020-05-07 se, sebastian(dot)ertz(at)tk(minus)ertz(dot)de
#
# Copyright (c) 2001-2020 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
# set global readonly vars
#------------------------------------------------------------------------------
# readonly DEBUG="yes"
readonly PKG_NAME=apache2-mod_brotli

# menu
readonly PKG_MENU_FILE=setup.services.apache2.config.modules.$PKG_NAME.menu

# files
readonly FILE_CONFIG=/etc/apache2/mods-available/brotli.conf


#==============================================================================
# common 20200405
#==============================================================================
. /var/install/include/eislib
_debug_pos=1
_debug_mecho(){
	[ "$DEBUG" ] && [ "$1" ] && mecho --link "$(printf "%*s" "$_debug_pos" " ")$*"
}
_debug_func(){
	_debug_mecho "$1() {}"
}
_debug_func_begin(){
	_debug_mecho "$1() {"
	_debug_pos=$((_debug_pos + 2))
}
_debug_func_end(){
	[ $_debug_pos -ge 3 ] && _debug_pos=$((_debug_pos - 2))
	_debug_mecho "} $1()"
}
_abort_config(){
	mecho --error "${1:-Abort config}!"
	exit 1
}
_rm_dir(){
	local name
	for name in "$@"; do
		if [ "${#name}" -ge 4 ]; then
			if [ "${name: -2}" = "/*" ] && [ "${#name}" -ge 6 ] && [ -d "${name:: -1}" ]; then
				_debug_mecho "rm -rf '$name'"
				name="${name:: -1}"
				rm -rf "$name"*
			fi
			name="${name%*/}/"
			if [ -d "$name" ]; then
				_debug_mecho "rmdir -p --ignore-fail-on-non-empty '$name'"
				rmdir -p --ignore-fail-on-non-empty "$name"
			fi
		fi
	done
}
_rm_file(){
	local name
	for name in "$@"; do
		if [ "$name" ] && [ -e "$name" ]; then
			_debug_mecho "rm -f '$name'"
			rm -f "$name"
		fi
	done
}
_fileheader_print(){
	printf "#----------------------------------------------------------------------------\n"
	printf "# %s\n" "$1"
	printf "# generated by %s\n" "$PGM_FULLNAME"
	printf "#\n"
	printf "# Do not edit this file, edit %s\n" "$PKG_FILE_CONFIG"
	printf "# Creation Date: %s Time: %s\n" "$EISDATE" "$EISTIME"
	printf "#----------------------------------------------------------------------------\n"
}
_fileheader_write(){
	if [ "$1" ]; then
		_debug_mecho "_fileheader_write ($1)"
		_fileheader_print "$1" > "$1"
	fi
}
_user_check(){
	if [ "$PKG_USER" ] && [ "$PKG_USER_HOME" ] && [ "$PKG_USER_SHELL" ] && [ "$PKG_GROUP" ]; then
		_debug_func_begin "${FUNCNAME[0]}"
		if ! getent passwd "$PKG_USER" >/dev/null 2>&1; then
			# add-user [-d|-l|-r] user encrypted-password uid group real-name home shell
			/var/install/bin/add-user -l -r "$PKG_USER" "*" "" "$PKG_GROUP" "${PKG_USER_COMMENT:-$PKG_USER}" "$PKG_USER_HOME" "$PKG_USER_SHELL" >/dev/null 2>&1
		else
			passwd -dl "$PKG_USER" >/dev/null 2>&1
			/var/install/bin/modify-user -g "$PKG_USER" "$PKG_GROUP" >/dev/null 2>&1
			[ "$PKG_USER_COMMENT" ] && /var/install/bin/modify-user -c "$PKG_USER" "$PKG_USER_COMMENT" >/dev/null 2>&1
			/var/install/bin/modify-user -d "$PKG_USER" "$PKG_USER_HOME" >/dev/null 2>&1
			/var/install/bin/modify-user -s "$PKG_USER" "$PKG_USER_SHELL" >/dev/null 2>&1
		fi
		chmod 0700 "$PKG_USER_HOME"
		chown -R "$PKG_USER":"$PKG_GROUP" "$PKG_USER_HOME"
		_debug_func_end "${FUNCNAME[0]}"
	fi
}
_cron_remove(){
	if [ "$FILE_CRON" ] && [ -f "$FILE_CRON" ]; then
		_debug_func_begin "${FUNCNAME[0]}"
		_rm_file "$FILE_CRON"
		/var/install/config.d/cron.sh --quiet
		_debug_func_end "${FUNCNAME[0]}"
	fi
}
eisfair_config_set_defaults(){ true;}
eisfair_config_print(){ true;}
do_install(){ true;}
_do_install(){
	_debug_func_begin "${FUNCNAME[0]}"
	[ -x "/etc/init.d/${PKG_NAME:?}" ] && "/etc/init.d/$PKG_NAME" --quiet stop
	_debug_func_begin "do_install"
	do_install
	_debug_func_end "do_install"
	[ "${_APACHE:-no}" = "yes" ] && _apache_eisfair_config_set_defaults
	if [ "${_COMMON_NO_EISFAIRCONF:-no}" != "yes" ]; then
		_debug_func "eisfair_config_set_defaults"
		eisfair_config_set_defaults
		_debug_func "eisfair_config_print"
		. /var/install/include/configlib
		{
			printgpl --conf "$PKG_NAME" "2020-06-11" "se"
			eisfair_config_print
			printend
		} > "$PKG_FILE_DEFAULT"
		chmod 0600 "$PKG_FILE_DEFAULT"
		chown root:root "$PKG_FILE_DEFAULT"
		if [ -f "$PKG_FILE_CONFIG" ]; then
			/var/install/bin/backup-file --quiet "$PKG_NAME"
			. "$PKG_FILE_CONFIG"
		fi
		{
			printgpl --conf "$PKG_NAME" "2020-06-11" "se"
			eisfair_config_print
			printend
		} > "$PKG_FILE_CONFIG"
		chmod 0600 "$PKG_FILE_CONFIG"
		chown root:root "$PKG_FILE_CONFIG"
	fi
	[ "$PKG_MENU_FILE_PARENT" ] && [ "$PKG_MENU_FILE" ] && [ "$PKG_MENU_NAME" ] && /var/install/bin/add-menu --menu "$PKG_MENU_FILE_PARENT" "$PKG_MENU_FILE" "$PKG_MENU_NAME"
	_do_config
	if [ -x "/etc/init.d/$PKG_NAME" ]; then
		local _start
		eval _start="\$START_${PKG_NAME^^}"
		if [ "${_start:-no}" = "yes" ]; then
			sleep 1
			"/etc/init.d/$PKG_NAME" start
		fi
	fi
	_debug_func_end "${FUNCNAME[0]}"
}
do_update(){ true;}
_do_update(){
	_debug_func_begin "${FUNCNAME[0]}"
	_cron_remove
	if [ "${_APACHE:-no}" = "yes" ]; then
		_apache_config_remove
		_apache_config_apply --quiet
	fi
	if [ -x "/etc/init.d/$PKG_NAME" ]; then
		if [ -f "$PKG_FILE_CONFIG" ] && grep -q -E "^START_${PKG_NAME^^}='yes'" "$PKG_FILE_CONFIG"; then
			"/etc/init.d/$PKG_NAME" stop
		else
			"/etc/init.d/$PKG_NAME" --quiet stop
		fi
	fi
	_rm_file \
		$PKG_FILE_DEFAULT \
		/etc/rc[0126S].d/[KS]??$PKG_NAME \
		$FILE_LOGROTATE
	[ "$PKG_MENU_FILE" ] && _rm_file "/var/install/menu/${PKG_MENU_FILE%*menu}"*
	_debug_func_begin "do_update"
	do_update
	_debug_func_end "do_update"
	_debug_func_end "${FUNCNAME[0]}"
}
do_uninstall(){ true;}
_do_uninstall(){
	_debug_func_begin "${FUNCNAME[0]}"
	_do_update
	[ "$PKG_MENU_FILE_PARENT" ] && [ "$PKG_MENU_FILE" ] && /var/install/bin/del-menu "$PKG_MENU_FILE_PARENT" "$PKG_MENU_FILE"
	if [ "$DIR_LOG" ] && [ -d "${DIR_LOG%*/}/" ]; then
		[ "$REGEX_LOGFILES" ] && _rm_file $(find "${DIR_LOG%*/}/" -maxdepth 1 -regextype posix-extended -regex  ".*$REGEX_LOGFILES.*" -type f 2>/dev/null)
		_rm_dir "${DIR_LOG%*/}/"
	fi
	[ "${_APACHE_DOCROOT:-no}" = "yes" ] && _apache_docroot_uninstall
	_rm_file "$PKG_FILE_CONFIG"
	_rm_file "$FILE_CONFIG"
	_rm_file "/etc/backup.d/$PKG_NAME."*
	_rm_dir "$DIR_PKG"
	if [ "$PKG_USER" ] && [ "$PKG_USER_HOME" ] && [ "$PKG_USER_SHELL" ] && [ "$PKG_GROUP" ]; then
		if [ "${_COMMON_USER_HOME_NO_DEL:-no}" != "yes" ]; then
			/var/install/bin/remove-user -f "$PKG_USER" y >/dev/null
		else
			/var/install/bin/remove-user -f "$PKG_USER" n >/dev/null
		fi
	fi
	_debug_func_begin "do_uninstall"
	do_uninstall
	_debug_func_end "do_uninstall"
	_debug_func_end "${FUNCNAME[0]}"
}
do_config(){ true;}
_do_config(){
	_debug_func_begin "${FUNCNAME[0]}"
	[ "$DIR_LOG" ] && [ ! -d "${DIR_LOG%*/}/" ] && mkdir -p "${DIR_LOG%*/}/"
	[ "${_APACHE:-no}" = "yes" ] && _apache_config_remove
	_rm_file /etc/rc[0126S].d/[KS]??"$PKG_NAME"
	_rm_file "$FILE_LOGROTATE"
	_cron_remove
	[ "${NO_DEL_FILE_CONFIG:-no}" != "yes" ] && _rm_file "$FILE_CONFIG"
	[ "$DIR_CRON" ] && [ ! -d "${DIR_CRON%*/}/" ] && mkdir -p "${DIR_CRON%*/}/"
	[ "$DIR_LOG"  ] && [ ! -d "${DIR_LOG%*/}/"  ] && mkdir -p "${DIR_LOG%*/}/"
	[ -f "$PKG_FILE_CONFIG" ] || _abort_config "${PKG_ECHO:-$PKG_NAME}: configuration file '$PKG_FILE_CONFIG' not found!"
	chmod 0600 "$PKG_FILE_CONFIG"
	chown root:root "$PKG_FILE_CONFIG"
	. "$PKG_FILE_CONFIG"
	_user_check
	_debug_func_begin "do_config"
	do_config
	_debug_func_end "do_config"
	local _start
	eval _start="\$START_${PKG_NAME^^}"
	if [ "${_start:-no}" = "yes" ]; then
		if [ -x "/etc/init.d/$PKG_NAME" ] && [ "$((PKG_INIT))" -ge 10 ]; then
				ln -sf "../init.d/$PKG_NAME" "/etc/rc2.d/S$((PKG_INIT))$PKG_NAME"
				ln -sf "../init.d/$PKG_NAME" "/etc/rc2.d/K$((100 - PKG_INIT))$PKG_NAME"
		fi
		if [ "$FILE_CRON" ] && [ -f "$FILE_CRON" ]; then
			mecho "${PKG_ECHO:-$PKG_NAME}: Cron apply ..."
			chmod 0644 "$FILE_CRON"
			chown root:root "$FILE_CRON"
			/var/install/config.d/cron.sh --quiet
			mecho --ok
		fi
	fi
	_debug_func_end "${FUNCNAME[0]}"
}
readonly -f _debug_mecho _debug_func _debug_func_begin _debug_func_end _abort_config _rm_dir _rm_file _fileheader_print _fileheader_write _user_check _cron_remove _do_install _do_update _do_uninstall _do_config
readonly PGM_FULLNAME="$0"
readonly PGM_BASENAME="$(basename "$PGM_FULLNAME")"
readonly PKG_FILE_CONFIG=/etc/config.d/$PKG_NAME
readonly PKG_FILE_DEFAULT=/etc/default.d/$PKG_NAME
#==============================================================================
# common END
#==============================================================================


#==============================================================================
# apache (CUSTOM)
#==============================================================================
readonly _APACHE=yes
readonly _APACHE_DIR_ENABLED=/etc/apache2/mods-enabled
readonly _APACHE_DIR_INCLUDE=/etc/apache2/mods-include
_APACHE_RELOAD="no"
_APACHE_RESTART="no"
_apache_eisfair_config_set_defaults(){ true;}
_apache_config_remove(){
	if [ -e "$_APACHE_DIR_ENABLED/brotli.conf" ] || [ -e "$_APACHE_DIR_ENABLED/brotli.load" ]; then
		_APACHE_RELOAD="yes"
		_APACHE_RESTART="yes"
	fi
	_rm_file \
		"$FILE_CONFIG" \
		"$_APACHE_DIR_ENABLED/brotli.conf" \
		"$_APACHE_DIR_ENABLED/brotli.load"
}
_apache_config_apply(){
	if [ "$_APACHE_RELOAD" = "yes" ] && [ -x "/var/install/config.d/apache2.sh" ]; then
		_APACHE_RESTART="yes"
		/var/install/config.d/apache2.sh
	fi
	if [ "$_APACHE_RESTART" = "yes" ] && [ -f "/run/apache2.pid" ] && [ -x "/etc/init.d/apache2" ]; then
		/etc/init.d/apache2 restart
	fi
	_APACHE_RELOAD="no"
	_APACHE_RESTART="no"
}
_do_apacheconf(){
	if [ -f $FILE_CONFIG ] && [ -f "/etc/apache2/mods-available/brotli.load" ]; then
		mecho "${PKG_ECHO:-$PKG_NAME}: activate"
		ln -sf "$FILE_CONFIG" "$_APACHE_DIR_ENABLED/brotli.conf"
		ln -sf "/etc/apache2/mods-available/brotli.load" "$_APACHE_DIR_ENABLED/brotli.load"
		mecho --ok
	else
		rm -f \
			"$_APACHE_DIR_ENABLED/brotli.conf" \
			"$_APACHE_DIR_ENABLED/brotli.load"
	fi
}
readonly -f _apache_eisfair_config_set_defaults _apache_config_remove _apache_config_apply _do_apacheconf
trap '_apache_config_apply;exit' INT TERM EXIT
#==============================================================================
# apache (CUSTOM) END
#==============================================================================


#==============================================================================
# eisfair_config_*
#==============================================================================
eisfair_config_set_defaults(){
	START_APACHE2_MOD_BROTLI="no"
	APACHE2_MOD_BROTLI_N="9"
	APACHE2_MOD_BROTLI_1_TYPE="application/javascript"
	APACHE2_MOD_BROTLI_2_TYPE="application/json"
	APACHE2_MOD_BROTLI_3_TYPE="application/rss+xml"
	APACHE2_MOD_BROTLI_4_TYPE="application/x-javascript"
	APACHE2_MOD_BROTLI_5_TYPE="text/css"
	APACHE2_MOD_BROTLI_6_TYPE="text/html"
	APACHE2_MOD_BROTLI_7_TYPE="text/javascript"
	APACHE2_MOD_BROTLI_8_TYPE="text/plain"
	APACHE2_MOD_BROTLI_9_TYPE="text/xml"
}
eisfair_config_print(){
	local _idx
	printgroup "Basic configuration"
	printvar "START_APACHE2_MOD_BROTLI"
	printvar "APACHE2_MOD_BROTLI_N"
	_idx=1
	while [ $_idx -le $((APACHE2_MOD_BROTLI_N)) ]; do
		printvar "APACHE2_MOD_BROTLI_${_idx}_TYPE"
		_idx=$((_idx + 1))
	done
}


#==============================================================================
# do_config
#==============================================================================
do_config(){
	if [ "${START_APACHE2_MOD_BROTLI:-no}" = "yes" ]; then
		local _idx
		local _type
		_APACHE_RELOAD="yes"
		_APACHE_RESTART="yes"
		mecho "$PKG_NAME: Write '$FILE_CONFIG' ..."
		{
			_fileheader_print "$FILE_CONFIG"
			printf "<IfModule !mod_filter.c>\n"
			printf "  LoadModule filter_module /usr/lib/apache2/modules/mod_filter.so\n"
			printf "</IfModule>\n"
			printf "<IfModule mod_brotli.c>\n"
			printf "  AddOutputFilterByType BROTLI_COMPRESS"
			_idx=1
			while [ $_idx -le $((APACHE2_MOD_BROTLI_N)) ]; do
				eval _type="\$APACHE2_MOD_BROTLI_${_idx}_TYPE"
				printf  " %s" "$_type"
				_idx=$((_idx + 1))
			done
			printf "\n"
			printf "  BrotliAlterEtag AddSuffix\n"
			printf "</IfModule>\n"
		} > "$FILE_CONFIG"
		chmod 0644 "$FILE_CONFIG"
		chown root:root "$FILE_CONFIG"
		mecho --ok
	fi
}


#==============================================================================
# Main
#==============================================================================
case "$1" in
	apacheconf ) _do_apacheconf ;;
	install    ) _do_install    ;;
	update     ) _do_update     ;;
	uninstall  ) _do_uninstall  ;;
	*          ) _do_config     ;;
esac


#==============================================================================
# End
#==============================================================================
exit 0
